// Copyright (c) 2008-2011 Done Smart Ltd (donesmart.com)
//
// This software is property of Done Smart Ltd,
// do not redistribute any part of it without our
// permission.
function SitelauncherSiteIcons()
{  
  this.sizeSmall = 16;
  this.sizeMedium = 20;
  this.sizeLarge = 24;
  this.sizeNameMap = ['iconDataSmall', 'iconData', 'iconDataLarge'];
  this.sizePixelsMap = [16,20,24];
  this.iconBuilderDoc = null;
  
  this.init = function()
  {
    sitelauncherSiteIcons.iconBuilderDoc = document.getElementById('sitelauncherIconBuilderIframe').contentDocument;
  };
  
  this.insertSiteIcons = function(site)
  {
    site.linkedURL = null; // linkedURL was persistent in older sl versions     
    sitelauncherSiteIcons.loadPixasticLibraryThenCall(
      function(){sitelauncherSiteIcons.retrieveIconFromWebsite(site)});
  };
  
  this.insertSiteIconsFromCustomURL = function(site, url)
  {
    site.linkedURL = url;
    sitelauncherSiteIcons.loadPixasticLibraryThenCall(
      function(){sitelauncherSiteIcons.retrieveIconFromWebsite(site)});
  };  

  this.insertSiteIconsFromDataURL = function(site, dataURL)
  {
    sitelauncherSiteIcons.loadPixasticLibraryThenCall(
      function()
      {
        var iconBuilderDoc = document.getElementById('sitelauncherIconBuilderIframe').contentDocument;
        var img = iconBuilderDoc.createElement('img');
        img.onload = function()
          {
            site.iconData = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeMedium);
            site.iconDataSmall = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeSmall);
            site.iconDataLarge = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeLarge); 
          
            sitelauncherHooks.fire('newSiteIconLoaded', site);
          };
        iconBuilderDoc.getElementsByTagName('body')[0].appendChild(img);        
        img.src = dataURL;
      });
  };

  this.loadPixasticLibraryThenCall = function(func)
  {
    sitelauncherUtilsDom.loadLibrary('pixastic', func);
  };
  
  this.reqObjects = []; 
  this.reqCount = -1;
  
  this.retrieveIconFromWebsite = function(site)
  {
    var url = null, src = null, baseURL, domain;
    
    if(site.linkedURL)
    {
      url = site.linkedURL; 
    }
    else if(site.type === sitelauncher.linkTypes.multi)
    {
      url = site.url.split('|')[0];
    } 
    else if(site.type === sitelauncher.linkTypes.search)
    {
      url = site.url.split('search:', 2)[1];  
    }
    else if(site.type === sitelauncher.linkTypes.javascript)
    {
      // no url to use
      return;
    }
    else
    {
      url = site.url;
    }         
    
    if(url === null || url === '')
    {
      return;
    }
    
    var isHttp = url.match(/^(http|https)+:\/\//i);
    var hasProtocol = url.match(/^[^:]+:/i);
    
    // we only support http(s) protocol
    if(hasProtocol && !isHttp)
    {
      return;
    }
    
    // if no protocol assume http
    if(!isHttp && !hasProtocol)
    {
      url = 'http://' + url;
    }
    
    baseURL = url.match(/[a-z0-9]+:\/\/[^\/]+/).toString();
    
    if(!baseURL)
    {
      return;
    }

    try
    {
      sitelauncherPrefs.indicatorShow('loading.gif', sitelauncherUtilsMisc.getLocaleString("requestingIcon"));
    }
    catch(e)
    {
    }
    
    // first check cache for icon location
    try
    {
      var faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
                             .getService(Components.interfaces.nsIFaviconService);
                       
      var uri = Components
                  .classes["@mozilla.org/network/simple-uri;1"]
                    .getService(Components.interfaces.nsIURI)

      uri.spec = url;
      var cached = faviconService.getFaviconForPage(uri).spec;
      
      if(cached)
      {
        src = cached;
      }
    }
    catch(e)
    {
    }
    
    if(src === null)
    {
      // avoid generic google icon for google calendar and reader
      
      if(url.match(/[a-z]+:\/\/calendar\.google\.com/i) || 
         url.match(/[a-z]+:\/\/(www\.)?google\.com\/calendar/i))
      {
        src = 'http://calendar.google.com/googlecalendar/images/favicon.ico';
      }
      else if(url.match(/[a-z]+:\/\/reader\.google\.com/i) || 
              url.match(/[a-z]+:\/\/(www\.)?google\.com\/reader/i))
      {
        src = 'http://www.google.com/reader/ui/favicon.ico';
      }
      else
      {
        src = baseURL + '/favicon.ico'
      }
    }
    
    var reqNum = ++sitelauncherSiteIcons.reqCount;
    
    sitelauncherSiteIcons.reqObjects[reqNum] = {req  : reqNum,
                                                site : site,
                                                baseURL : baseURL + '/',
                                                src : src,
                                                img  : null,
                                                browser : null,
                                                iconURL : null,
                                                isRetry : false};
    
    sitelauncherSiteIcons.abortIfTakingTooLong(reqNum);
    sitelauncherSiteIcons.initRequestForIconImage(sitelauncherSiteIcons.reqObjects[reqNum]);
  };
  
  this.abortIfTakingTooLong = function(reqNum)
  {
    setTimeout(function(e)
      {
        var req = sitelauncherSiteIcons.reqObjects[reqNum];
        
        if(!req)
        {
          return;
        }
        
        if(req.img && req.img.parentNode)
        {
          req.img.parentNode.removeChild(req.img);
        }
        
        if(req.browser)
        {
          var browser = req.browser;
          browser.stop();
          browser.destroy();
          if(browser.parentNode)
          {
            browser.parentNode.removeChild(browser);
          }
        }
        
        delete sitelauncherSiteIcons.reqObjects[reqNum];    
        
        try
        {
          sitelauncherPrefs.indicatorHide();  
        }catch(e)
        {
        }
        
      }, 10000);
  },
  
  this.retrieveIconFromDocument = function(req)
  {
    var browser = document.createElement('browser');
    browser.setAttribute('type', 'content-targetable');  
    browser.setAttribute('autocompleteenabled', 'false');
    browser.setAttribute('collapsed', 'true');
    browser.setAttribute('style', 'width:0 !important;');
    browser.setAttribute('__slReqNum',  req.req);
    
    document.documentElement.appendChild(browser);

    browser.webNavigation.allowMetaRedirects = true;
    browser.webNavigation.allowJavascript = false;
    browser.webNavigation.allowPlugins = false;
    browser.webNavigation.allowSubframes = false;
    browser.webNavigation.allowImages = false;
    browser.webNavigation.allowAuth = false;
    
    browser.webNavigation.loadURI(req.baseURL, 
                                  Components.interfaces.nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY,
                                  null, null, null);   
                                  
    req.browser = browser;
                        
    browser.addEventListener("DOMContentLoaded", 
      function(e){sitelauncherSiteIcons.handleBrowserContentLoaded(e);}, true);
  };
  
  
  this.handleBrowserContentLoaded = function(e)
  {
    var doc = e.originalTarget;
    var browser = e.currentTarget;
    
    if(!browser.hasAttribute('__slReqNum'))
    {
      return;
    }
    
    var reqNum = browser.getAttribute('__slReqNum');
    var req = sitelauncherSiteIcons.reqObjects[reqNum];
    var iconURL = sitelauncherSiteIcons.getIconLocationFromDoc(doc);
    
    if(req && iconURL)
    {
      iconURL = sitelauncherSiteIcons.getIconLocationFromDoc(doc);
      req.iconURL = iconURL;
      sitelauncherSiteIcons.initRequestForIconImage(req); 
    }
    
    browser.destroy();
    
    if(browser.parentNode)
    {
      browser.parentNode.removeChild(browser);
    }
    
    if(req)
    {
      req.browser = null;
    }
  };
  
  this.getIconLocationFromDoc = function(doc)
  {
    if(!doc || doc.nodeName !== "#document" || !doc.getElementsByTagName)
    {
      return false;
    }

    var nodes = doc.getElementsByTagName('link');
    
    for(var x = 0; x < nodes.length; x++)
    {
      var relAttr, values = [];
      
      if(typeof (relAttr = nodes[x].getAttribute('rel')) === 'string')
      {
        values = relAttr.toLowerCase().split(/\s+/);
      }
      
      if(values.indexOf('icon') > -1 && values.indexOf('shortcut') > -1)
      {
        return nodes[x].href;
      }
    }
    
    var url = doc.location.href.match(/[a-z0-9]+:\/\/[^\/]+/);
    if(url)
    {
      return url + '/favicon.ico';
    }
    
    return false;
  };
  
  this.initRequestForIconImage = function(req)
  {
    var iconBuilderDoc = document.getElementById('sitelauncherIconBuilderIframe').contentDocument;
    
    var img = iconBuilderDoc.createElement('img');
    img.setAttribute('__slReqNum',  req.req);
    
    img.onload = sitelauncherSiteIcons.handleImageLoadEvent;
    img.onerror = sitelauncherSiteIcons.handleImageErrorEvent;
    iconBuilderDoc.getElementsByTagName('body')[0].appendChild(img);
    req.img = img;
    req.img.src = req.iconURL || req.src;
  },
  
  this.handleImageLoadEvent = function(e)
  {
    var img = e.target;
    
    if(!img.hasAttribute('__slReqNum'))
    {
      return;
    }
    
    var reqNum = img.getAttribute('__slReqNum');
    var req = sitelauncherSiteIcons.reqObjects[reqNum];    
     
    if(req && req.site)
    {
      var site = req.site;
      site.iconData = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeMedium);
      site.iconDataSmall = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeSmall);
      site.iconDataLarge = sitelauncherSiteIcons.generateIconFromImage(img, sitelauncherSiteIcons.sizeLarge); 
      
      sitelauncherHooks.fire('newSiteIconLoaded', site);
    }
    
    try
    {
      sitelauncherPrefs.indicatorHide();   
    }
    catch(e)
    {
    }

    if(img.parentNode)
    {
      img.parentNode.removeChild(img);  
    }
    
    if(req)
    {
      req.img = null; 
      delete sitelauncherSiteIcons.reqObjects[req.req];
    }
  };
  
  this.handleImageErrorEvent = function(e)
  {
    var img = e.target;
    
    if(!img.hasAttribute('__slReqNum'))
    {
      return;
    }
    
    var reqNum = img.getAttribute('__slReqNum');
    var req = sitelauncherSiteIcons.reqObjects[reqNum];

    if(img.parentNode)
    {
      img.parentNode.removeChild(img);  
    }
    
    if(!req)
    {
      return;
    }
    
    req.img = null;

    if(!req.isRetry && req.site)
    {
      req.isRetry = true;
      sitelauncherSiteIcons.retrieveIconFromDocument(req);
    }
    else
    {
      delete sitelauncherSiteIcons.reqObjects[req.req];
      
      try
      {
        sitelauncherPrefs.indicatorHide();
      }catch(e)
      {
      }
    }
  };
  
  this.generateIconFromImage = function(img, pixels)
  {
    var canvas = sitelauncherSiteIcons.iconBuilderDoc.createElement('canvas');
    canvas.height = pixels + pixels / 2;
    canvas.width = pixels;
    var context = canvas.getContext("2d");
    context.drawImage(img, 0, 0, pixels, pixels);
    context.translate(0, pixels * 2);
    context.scale(1, -1);
    context.drawImage(img, 0, 0, pixels, pixels);
    context.globalCompositeOperation = "destination-out";
    var gradient = context.createLinearGradient(0, 0, 0, pixels);
    gradient.addColorStop(.3, "rgba(255, 255, 255, 1)");
    gradient.addColorStop(1, "rgba(255, 255, 255, .5)");
    context.fillStyle = gradient;
    context.fillRect(0, 0, pixels, pixels);
    
    if(sitelauncherSiteIcons.sizeLarge === pixels && img.width < pixels)
    {
      // sharpen enlarged icon
      var Pixastic = sitelauncherUtilsDom.getLibrary('pixastic').Pixastic;
      var newcanvas = Pixastic.process(canvas, "unsharpmask", {amount:90, radius:5, threshold:2}); 
      // max: 500, 5, 255
    }
    else
    {
      var newcanvas = canvas;
    }
    
    return newcanvas.toDataURL("image/png", "");
  };  
  
  this.isSiteIconBlank = function(site)
  {
    var requiredIconSize = sitelauncher.requiredIconSize !== -1 ? sitelauncher.requiredIconSize : 1;
    return site[sitelauncherSiteIcons.sizeNameMap[requiredIconSize]] === sitelauncherSiteIcons.getBlankIcon();
  };
  
  this.getBlankIcon = function()
  {
    var requiredIconSize = sitelauncher.requiredIconSize !== -1 ? sitelauncher.requiredIconSize : 1;
    return sitelauncher[['defaultPageIconSmall', 'defaultPageIcon', 'defaultPageIconLarge'][requiredIconSize]];
  };

  this.getSiteURL = function()
  {
    
  };

  this.getSiteIconDimensions = function()
  {
    if(sitelauncher.getBoolPref('extensions.sitelauncher.launcher.showIconReflections'))
    {
      return sitelauncherSiteIcons.getSiteIconDimensionsWithReflection();
    }
    else
    {
      return sitelauncherSiteIcons.getSiteIconDimensionsWithoutReflection();
    }
  };
  
  this.getSiteIconDimensionsWithAttributes = function(size, reflection)
  {
    var width = sitelauncherSiteIcons.sizePixelsMap[size];
    
    if(reflection) 
    {
      return [width, width + (width / 2)];
    }
    else 
    {
      return [width, width];
    }
  };
  
  this.getSiteIconDimensionsWithReflection = function()
  {
    var width = sitelauncherSiteIcons.sizePixelsMap[sitelauncher.getIntPref('extensions.sitelauncher.launcher.iconSize')];
    return [width, width + (width / 2)];
  };  
  
  this.getSiteIconDimensionsWithoutReflection = function()
  {
    var width = sitelauncherSiteIcons.sizePixelsMap[sitelauncher.getIntPref('extensions.sitelauncher.launcher.iconSize')];
    return [width, width];
  };    

  this.getSiteIcon = function(site)
  {
    return sitelauncherSiteIcons.getSiteIconBySize(site, sitelauncher.getIntPref('extensions.sitelauncher.launcher.iconSize'));
  };

  this.getSiteIconBySize = function(site, size)
  {
    return site[sitelauncherSiteIcons.sizeNameMap[size]];
  };

  this.backupOriginalIconData = function(site)
  {
    // don't do anything if original icon has already been processed
    if(site.isUsingCustomIcon)
    {
      return; 
    }
    
    // only purposeful to back up if we have all icon sizes
    if(site.iconDataSmall !== sitelauncher.defaultPageIconSmall &&
       site.iconData !== sitelauncher.defaultPageIcon && 
       site.iconDataLarge !== sitelauncher.defaultPageIconLarge)
    {
      site.originalIconDataSmall = site.iconDataSmall;  
      site.originalIconData = site.iconData;
      site.originalIconDataLarge = site.iconDataLarge;         
    }
    
    site.isUsingCustomIcon = true;
  };
  
  this.restoreOriginalIconData = function(site)
  {
    if(site.isUsingCustomIcon && 
       site.originalIconData !== "" && 
       site.originalIconDataSmall !== "" && 
       site.originalIconDataLarge !== "")
    {
      site.iconDataSmall = site.originalIconDataSmall;  
      site.iconData = site.originalIconData;
      iconDataLarge = site.originalIconDataLarge;  
      sitelauncherHooks.fire('newSiteIconLoaded', site);
    } 
    else
    {
      // original not cached - refetch it
      sitelauncherSiteIcons.insertSiteIcons(site); 
    }  
    
    site.isUsingCustomIcon = false;    
    site.originalIconData = "";
    site.originalIconDataSmall = "";
    site.originalIconDataLarge = "";
  };
}
var sitelauncherSiteIcons = new SitelauncherSiteIcons();

function SitelauncherSiteIconDataMigrator()
{
  this.migrateAllSiteIcons = function()
  {
    sitelauncherSiteIcons.loadPixasticLibraryThenCall(sitelauncherSiteIconDataMigrator.__migrateAllSiteIcons);
  };

  this.__migrateAllSiteIcons = function()
  {
    var sites = sitelauncher.sites;
    
    var iconBuilderDoc = sitelauncherSiteIcons.iconBuilderDoc;
    var body = iconBuilderDoc.getElementsByTagName('body')[0];
    
    for(var x = 0; x < sites.length; x++)
    {
      var site = sites[x];
      
      if(site.iconData === sitelauncher.defaultPageIcon)
      {
        site.iconDataSmall = sitelauncher.defaultPageIconSmall;
        site.iconDataLarge = sitelauncher.defaultPageIconLarge;
        site.iconsMigrated = true;      
        continue;  
      }
            
      var img = iconBuilderDoc.createElement('img');
      
      sitelauncherUtilsDom.linkElmToSite(img, site)
      
      img.addEventListener('load', sitelauncherSiteIconDataMigrator.useImageObjectToUpgradeIconData, false);
      img.src = site.iconData;        
      
      body.appendChild(img);
    }    
  }
  
  this.useImageObjectToUpgradeIconData = function(e)
  {
    var img = e.target;
    
    img.removeEventListener('load', sitelauncherSiteIconDataMigrator.useImageObjectToUpgradeIconData, false);

    var site = sitelauncherUtilsDom.getSiteByElm(img); 

    if(site && !site.iconsMigrated)
    {
      var canvas = sitelauncherSiteIcons.iconBuilderDoc.createElement('canvas');
      canvas.height = 20;
      canvas.width = 20;
      var context = canvas.getContext("2d");
      context.drawImage(img, 0, 0, 20, 30); // will crop reflection 
  
      site.iconDataSmall = sitelauncherSiteIcons.generateIconFromImage(canvas, sitelauncherSiteIcons.sizeSmall);
      site.iconDataLarge = sitelauncherSiteIcons.generateIconFromImage(canvas, sitelauncherSiteIcons.sizeLarge);
      site.iconsMigrated = true;
    }

    img.parentNode.removeChild(img); 
    
    sitelauncherSiteIconDataMigrator.applyChangesIfAllIconDataMigrated();
  };  
  
  this.applyChangesIfAllIconDataMigrated = function()
  {
    for(var x = 0; x < sitelauncher.sites.length; x++)
    {
      if(!sitelauncher.sites[x].iconsMigrated)
      {
        return;
      }
    }

    sitelauncher.shortcutSchemaVersion = 2;
    sitelauncher.save();
    sitelauncherLauncher.update();
    sitelauncherLauncher.buffer();
    
    sitelauncherHooks.fire('iconsMigrated', true);
  };  
};
var sitelauncherSiteIconDataMigrator = new SitelauncherSiteIconDataMigrator();

function SitelauncherPanels()
{
  this.panels = {};
  
  this.add = function(id, contents)
  {
    contents.setAttribute('id', id);
    this.panels[id] = {'ref'            : contents,
                       'width'          : 0,
                       'height'         : 0,
                       'screenX'        : null,
                       'screenY'        : null,
                       'onshown'        : null,
                       'minWidth'       : null,
                       'minHeight'      : null,
                       'bgColor'        : null,
                       'bgOpacity'      : null,
                       'textColor'      : null,
                       'roundedCorners' : null,
                       'ignoreKeys'     : null,
                       'shown'          : false,
                       'ready'          : false};
  };
  
  this.setScreenX = function(id, x)
  {
    this.panels[id].screenX = x;
  };
  
  this.setScreenY = function(id, y)
  {
    this.panels[id].screenY = y;
  };

  this.setOnShown = function(id, func)
  {
    this.panels[id].onshown = func;
  };
  
  this.setMinWidth = function(id, width)
  {
    this.panels[id].minWidth = width;
  };
  
  this.setMinHeight = function(id, width)
  {
    this.panels[id].minHeight = height;
  };
  
  this.setBgColor = function(id, color)
  {
    this.panels[id].bgColor = color;
  };
  
  this.setTextColor = function(id, color)
  {
    this.panels[id].textColor = color;
  };
  
  this.setBgOpacity = function(id, opacity)
  {
    this.panels[id].bgOpacity = opacity;
  };
  
  this.setRoundCorners = function(id, state)
  {
    this.panels[id].roundedCorners = state;
  };
  
  this.setIgnoreKeys = function(id, state)
  {
    this.panels[id].ignoreKeys = state;
  };
  
  this.isShowing = function(id)
  {
    return this.panels[id] ? this.panels[id].shown : false;
  };
  
  this.exists = function(id)
  {
    return this.panels[id] ? true : false;
  };
  
  this.toggle = function(id)
  {
    var el = this.panels[id];
    if(el && el.ready)
    {
      if(!el.shown)
      {
        this.show(id);
      }
      else
      {
        this.hide(id);
      }
    }     
  };

  this.show = function(id)
  {
    var el = this.panels[id];
    if(el && el.ready && !el.shown)
    {
      el.ready = false;
      el.shown = true;
      var panel = document.createElement('popup');
      var box = document.createElement('box');
      panel.setAttribute('class', 'sitelauncherPanel');
      panel.setAttribute('__sitelauncherPanel', id);
      panel.setAttribute('align', 'start');
      panel.setAttribute('ignorekeys', el.ignoreKeys !== null ? (el.ignoreKeys ? 'true' : 'false') : 'true');
      
      panel.addEventListener('popuphidden', 
        function(e)
        {
          // todo: rewrite to avoid closure
          sitelauncherPanels.cleanUp(id);
        }, false);
      
      if(el.onshown !== null)
      {
        panel.addEventListener('popupshown', el.onshown, false);
      }
      
      box.style.setProperty('background-color', 'transparent', null);
      box.style.setProperty('height', el.height + 'px', null);
      box.appendChild(el.ref);
      panel.appendChild(box);
      document.documentElement.appendChild(panel);
      
      if(id === 'sitelauncherLauncherMenuBox') // todo: abstract
      {        
        var lpanel = sitelauncherLauncher.nodes.panel;
        var panelY = lpanel.boxObject.screenY + lpanel.boxObject.height;
        var panelX = parseInt((lpanel.boxObject.screenX + lpanel.boxObject.width / 2) - (el.width / 2), 10);
        
        panel.showPopup(document.documentElement, panelX, panelY, 'context', null, null);
      }
      else
      {
        var screenX = (parseInt(document.width, 10) / 2) - (el.width / 2);
        var screenY = (parseInt(document.height, 10) / 2) - (el.height / 2);      
  
        screenX += document.documentElement.boxObject.screenX;
        screenY += document.documentElement.boxObject.screenY;
        
        panel.showPopup(document.documentElement, screenX, screenY, 'context', null, null);
        
        
      }
             
      el.ready = true;
    }   
  };
  
  this.hide = function(id)
  {
    var el = this.panels[id];
    if(el && el.ready && el.shown)
    {
      el.ready = false;
      el.shown = false;
      sitelauncherPanels.cleanUp(id);
    }
  };
  
  this.cleanUp = function(id)
  {
    var el = sitelauncherPanels.panels[id];
    if(el)
    {
      var done = document.getElementsByAttribute('__sitelauncherPanel', id);
      if(done && done[0])
      {
        done = done[0];
        done.parentNode.removeChild(done);
        el.shown = false;      
        el.ready = true;
      }
    }
  };  
 
  this.buffer = function(id)
  {
    var panel = sitelauncherPanels.panels[id];
    
    if(!panel || !panel.ref)
    {
      return;
    }
    
    var iframe = document.getElementById('sitelauncherBufferFrame');
    iframe.contentDocument.documentElement.setAttribute('align', 'start');
    
    // remove leftover node
    var el = iframe.contentDocument.getElementById('buffer__' + id); 
    if(el)
      el.parentNode.removeChild(el);
        
    var container = iframe.contentDocument.createElement('vbox');
    container.setAttribute('id', 'buffer__' + id);
        
    panel.ref.style.setProperty(
      'font-size', 
      ['small','medium','large'][sitelauncher.getIntPref('extensions.sitelauncher.launcher.fontSize')], 
      '');          
        
    var clone = iframe.contentDocument.importNode(panel.ref, true);
    container.appendChild(clone);
    iframe.contentDocument.documentElement.appendChild(container);

    panel.width = Math.max(clone.boxObject.width, panel.minWidth);
    panel.height = Math.max(clone.boxObject.height, panel.minHeight);
    panel.ref.style.setProperty('width', panel.width + 'px', null);

    var bgColor = panel.bgColor !== null ? 
                    panel.bgColor : sitelauncher.getCharPref('extensions.sitelauncher.launcher.color.background');
    
    var opacity = panel.bgOpacity !== null ?
                    panel.bgOpacity : sitelauncher.getIntPref('extensions.sitelauncher.launcher.opacity') / 10;
    
    var roundedCorners = panel.roundedCorners !== null ? 
                           panel.roundedCorners : sitelauncher.getBoolPref('extensions.sitelauncher.launcher.roundedCorners');
                         
    var gradient = sitelauncher.getIntPref('extensions.sitelauncher.launcher.gradient');                      
                             
    var bgImg = sitelauncherUtilsMisc.createBackgroundImage(
                  panel.width, panel.height, 
                  bgColor, opacity, 0, roundedCorners ? 5 : false, true);
        
    panel.ref.style.setProperty('background', 'url("' +  bgImg + '") 0px 0px no-repeat', null); 
    
    panel.ready = true;
  };
 
  // calculate dimensions and generate background images
  this.bufferAll = function()
  {
    for(var x in this.panels)
    {
      this.panels[x].ready = false;
    }
    
    for(var x in this.panels)
    {
      this.buffer(x);
    }
  };
};
var sitelauncherPanels = new SitelauncherPanels();

function SitelauncherUtilsDom()
{
  this.getIntAttr = function(elm, attr, defaultval)
  {
    var val = elm.getAttribute(attr); 
    
    // gecko implemntation of getAttribute returns "" if attr not set
    if(val !== '') 
    {
      return parseInt(val, 10);
    }   
    
    return defaultval;
  };
  
  this.getBoolAttr = function(elm, attr, defaultval)
  {
    var val = elm.getAttribute(attr); 
    
    // gecko implemntation of getAttribute returns "" if attr not set
    if(val !== '') 
    {
      return val === "true";
    }   
    
    return defaultval;  
  };
  
  this.getIntAttrTrusted = function(elm, attr)
  {
    return parseInt(elm.getAttribute(attr), 10);
  };
  
  this.getStrAttr = function(elm, attr, defaultval)
  {
    if(elm.hasAttribute(attr))
    {
      return elm.getAttribute(attr);
    }
    
    return defaultval;
  };
  
  // only use when sure attr exists
  this.getStrAttrTrusted = function(elm, attr)
  {
    return elm.getAttribute(attr);
  };

  this.getFirstElmByAttr = function(parent, attrName, attrValue)
  {
    var items = parent.getElementsByAttribute(attrName, attrValue);
    var item = items[0];
    
    if(item)
    {
      if(item.getAttribute(attrName) === attrValue.toString())
      {
        return item;
      }
    }
    
    return null;
  };

  this.idsOfLoadedLibraries = [];
  this.loadedLibraries = [];
  this.loadLibrary = function(libname, callback)
  {
    if(sitelauncherUtilsDom.idsOfLoadedLibraries.indexOf(libname) > -1)
    {
      callback();
      return;
    }
    
    var iframe = document.createElement('iframe');
    iframe.setAttribute('id', 'sitelauncherIframe_' + libname);
    iframe.setAttribute('style', 'width:0px;');
    iframe.setAttribute('collapsed', 'true');
    document.documentElement.appendChild(iframe);

    iframe.contentWindow.location = 'chrome://sitelauncher/content/lib/' + libname + '.html';
    
    iframe.contentWindow.onload = 
      function(e)
      {
        sitelauncherUtilsDom.idsOfLoadedLibraries.push(libname);
        sitelauncherUtilsDom.loadedLibraries[libname] = iframe.contentWindow;
        
        if(callback)
        {
          callback();
        }
      }
  };
  
  this.getLibrary = function(name)
  {
    return sitelauncherUtilsDom.loadedLibraries[name];
  };
  
  this.getSiteByElm = function(elm)
  {
    return sitelauncher.getSiteById(elm.getAttribute('__slItemId'));  
  };
  
  this.linkElmToSite = function(elm, site)
  {
    elm.setAttribute('__slItemId', site.id);
  };

  this.deleteAllChildNodes = function(node)
  {
    while(node.hasChildNodes())
    {
      node.removeChild(node.firstChild);
    }
  };
  
  this.animateNodeOutline = function(node, color_hex, duration_sec)
  {
    node.style.setProperty("outline", "1px solid " + color_hex, null);
    
    setTimeout(function(e)
      {
        node.style.setProperty("outline", "", null);
      }, duration_sec * 1000);
  };
  
  this.createLabel = function(value, node_attributes)
  {
    var label = document.createElement('label');

    if(sitelauncherLauncher.OS === 'WINNT')
    {
      label.setAttribute('value', '\u200D' + value + '\u200D'); // fix for vista cleartype bug  
    }
    else
    {
      label.setAttribute('value', value);
    }
    
    if(typeof node_attributes !== "undefined") 
    {
      for(var x in node_attributes) 
      {
        if(x === "class" || x === "end" || x === "style" || x === "crop") 
        {
          label.setAttribute(x, node_attributes[x]);
        }
      }
    }
    
    return label;
  };
}
var sitelauncherUtilsDom = new SitelauncherUtilsDom();

function SitelauncherHashTables()
{
  this.items = {};
  
  this.create = function(table, items, index)
  {
    for(var x = 0; x < items.length; x++)
    {
      var item = items[x];
      
      if(item[index] !== null && item[index] !== '')
      {
        this.items[table + item[index]] = item;
      }
    }
  };
  
  this.insert = function(table, id, item)
  {
    this.items[table + id] = item; 
  };
  
  this.get = function(table, id)
  {
    if(typeof this.items[table + id] !== 'undefined')
    {
      return this.items[table + id];
    }
    
    return null;
  };
  
  this.getCaseInsensitive = function(table, id)
  {
    var lcase = id.toLowerCase();
    if(typeof this.items[table + lcase] !== 'undefined')
    {
      return this.items[table + lcase];
    }
    
    var ucase = id.toUpperCase();
    if(typeof this.items[table + ucase] !== 'undefined')
    {
      return this.items[table + ucase];
    }
    
    return null;
  };  
  
  this.remove = function(table, id)
  {
    if(typeof this.items[table + id] !== 'undefined')
    {
      delete this.items[table + id];
    }
  };

  this.clearAll = function()
  {
    this.items = {};
  };
}
var sitelauncherHashTables = new SitelauncherHashTables();

function SitelauncherUtilsMisc()
{
  this.showPrefManageView = function()
  {
    var prefWindow = this.showPaneInPrefWindowInstance('sitelauncherPrefPaneManage'); 
    
    if(!prefWindow)
    {
      window.openDialog('chrome://sitelauncher/content/prefs.xul', 
                        'Manage shortcuts', 
                        'chrome,titlebar,toolbar,modal,resizable,close=no,top=0,left=0',
                        'sitelauncherPrefPaneManage');  
    }
  };
  
  this.showPrefShortcutPageView = function(url, content_title)
  {
    var url = url || gBrowser.selectedBrowser.currentURI.spec;
    var title = typeof content_title !== "undefined" ? content_title : gBrowser.selectedBrowser.contentTitle;
    
    var prefWindow = this.showPaneInPrefWindowInstance('sitelauncherPrefPaneManage');  
    
    if(!prefWindow)
    {    
      window.openDialog('chrome://sitelauncher/content/prefs.xul', 
                        'Manage shortcuts', 
                        'chrome,titlebar,toolbar,modal,resizable,close=no,top=0,left=0',
                        'sitelauncherPrefPaneManage',
                        url,
                        title);  
    }
    else
    {
      prefWindow.sitelauncherPrefs.populateAddFormWithURL(url, title);     
    }
  };  
  
  this.showPrefOptionsView = function()
  {
    var prefWindow = this.showPaneInPrefWindowInstance('sitelauncherPrefPaneOptions');     
    
    if(!prefWindow)
    {       
      window.openDialog('chrome://sitelauncher/content/prefs.xul', 
                        'Manage shortcuts', 
                        'chrome,titlebar,toolbar,modal,resizable,close=no,top=0,left=0',
                        'sitelauncherPrefPaneOptions');  
    }
  };
  
  this.showPrefHelpView = function()
  {
    var prefWindow = this.showPaneInPrefWindowInstance('sitelauncherPrefPaneHelp');    
    
    if(!prefWindow)
    {      
      window.openDialog('chrome://sitelauncher/content/prefs.xul', 
                        'Manage shortcuts', 
                        'chrome,titlebar,toolbar,modal,resizable,close=no,top=0,left=0',
                        'sitelauncherPrefPaneHelp');      
    }
  };  
  
  this.showPrefMoreView = function()
  {
    var prefWindow = this.showPaneInPrefWindowInstance('sitelauncherPrefPaneMore');    
    
    if(!prefWindow)
    {      
      window.openDialog('chrome://sitelauncher/content/prefs.xul', 
                        'Manage shortcuts', 
                        'chrome,titlebar,toolbar,modal,resizable,close=no,top=0,left=0',
                        'sitelauncherPrefPaneMore');      
    }
  };    
  
  this.showPaneInPrefWindowInstance = function(paneId)
  {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
    var prefWindow = wm.getMostRecentWindow("sitelauncher:options");
    
    if(!prefWindow)
    {
      return false;
    }
    
    prefWindow.document.documentElement.showPane(prefWindow.document.getElementById(paneId));
    prefWindow.focus();
      
    return prefWindow;
  };

  this.getKeyComboAsLocaleText = function(keys)
  {
    var output = [];
    for(var x = 0; x < keys.length; x++)
    {
      if(keys[x] > 0) // ignore unassigned keys
      {
        output.push(sitelauncherUtilsMisc.getKeyAsLocaleText(keys[x]));
      }
    }  
    
    return output.join(' + ');
  };

  this.getKeyAsLocaleText = function(id)
  {
    return sitelauncherUtilsMisc.getLocaleString('keylabel' + id);
  };

  this.getLocaleString = function(id)
  {
    var strbundle = document.getElementById("sitelauncherStrings");
  
    return strbundle.getString(id);
  };

  this.createBackgroundImage = function(width, height, bgcolor, opacity, gradient, roundedCorners, stroke)
  {
    var canvas = document.getElementById("sitelauncherCanvasFrame").contentDocument.createElement("canvas");
    canvas.width = width;
    canvas.height = height;
    var ctx = canvas.getContext("2d");
    var rgb = [parseInt(bgcolor.substring(1, 3), 16), 
               parseInt(bgcolor.substring(3, 5), 16), 
               parseInt(bgcolor.substring(5, 7), 16)];

    if(gradient)
    { 
      var add = rgb[0] + rgb[1] + rgb[2] / 3 > 127.5 ? -50 * gradient : 50 * gradient;
      var gradientStyle = ctx.createLinearGradient(0, 0, 0, height);
      gradientStyle.addColorStop(0,"rgba(" + rgb[0] +  "," + rgb[1] + ',' + rgb[2] + ", " + opacity + ")"); 
      gradientStyle.addColorStop(1,"rgba(" + 
        Math.min(Math.max(rgb[0] + add, 0), 255) + "," + 
        Math.min(Math.max(rgb[1] + add, 0), 255) + ',' + 
        Math.min(Math.max(rgb[2] + add, 0), 255) + ", " + opacity + ")"); 
      ctx.fillStyle = gradientStyle;
    }
    else
    {
      ctx.fillStyle = "rgba(" + rgb[0] +  "," + rgb[1] + ',' + rgb[2] + ", " + opacity + ")";
    }

    var borderWidth = stroke ? 1 : false;
    var x = 0;
    var y = 0;

    if(roundedCorners)
    {
      var radius = roundedCorners;

      ctx.beginPath();
      ctx.moveTo(x, y + radius);
      ctx.lineTo(x, y + height - radius);
      ctx.quadraticCurveTo(x, y + height, x + radius, y + height);
      ctx.lineTo(x + width - radius, y + height);
      ctx.quadraticCurveTo(x + width, y + height, x + width, y + height - radius);
      ctx.lineTo(x + width, y + radius);
      ctx.quadraticCurveTo(x + width, y, x + width - radius, y);
      ctx.lineTo(x + radius, y);
      ctx.quadraticCurveTo(x, y, x, y + radius);
      ctx.closePath();
      ctx.fill();    
    }
    else
    {
      ctx.fillRect(0, 0, width, height);
    }

    return canvas.toDataURL("image/png", "");
  };
}
var sitelauncherUtilsMisc = new SitelauncherUtilsMisc();

function SitelauncherUtilsFile()
{
  this.fileToDataURL = function(file, validTypes)
  {
    try
    {
      var type = Components.classes["@mozilla.org/mime;1"].getService(Components.interfaces.nsIMIMEService).getTypeFromFile(file);
  
      if(validTypes.indexOf(type) === -1)
      {
        return null;
      }
  
      var is = Components.classes["@mozilla.org/network/file-input-stream;1"].
                 createInstance(Components.interfaces.nsIFileInputStream);
  
      is.init(file, 0x01, 0, 0);   
      
      var bs = Components.classes["@mozilla.org/binaryinputstream;1"].
                 createInstance(Components.interfaces.nsIBinaryInputStream);
  
      bs.setInputStream(is);             
      
      var data = btoa(bs.readBytes(bs.available()));
    }
    catch(e)
    {
      return null;
    }
    
    return "data:" + type + ";base64," + data;
  };
}
var sitelauncherUtilsFile = new SitelauncherUtilsFile();

function SitelauncherHooks()
{
  this.attached = {};
  
  this.attach = function(hook, func)
  {
    if(typeof this.attached[hook] === 'undefined')
    {
      this.attached[hook] = [];
    }
    
    this.attached[hook].push(func);
  };
  
  this.fire = function(hook, detail)
  {
    if(typeof this.attached[hook] === 'undefined')
    {
      return;
    }
    
    for(var x = 0; x < this.attached[hook].length; x++)
    {
      this.attached[hook][x](detail);
    }
  };  
}
var sitelauncherHooks = new SitelauncherHooks();