// Copyright (c) 2008-2012 Done Smart Ltd (donesmart.com)
//
// This software is property of Done Smart Ltd,
// do not redistribute any part of it without our
// permission.
var sitelauncher = 
{
  sites : [], 
  groups : [], 
  prefs : null, 
  linkTypes : {
    standard : 0,
    javascript : 1,
    multi : 2,
    launchmod : 3,
    search : 4
  },
  prefValues : {},
  defaultPageIcon : 'chrome://sitelauncher/content/icons/default',
  defaultPageIconSmall : 'chrome://sitelauncher/content/icons/defaultSmall.png',  
  defaultPageIconLarge : 'chrome://sitelauncher/content/icons/defaultLarge.png',   
  nextUniqueId : 1,
  shortcutSchemaVersion : 1,
  requiredIconSize: -1,
  
  getUniqueId : function()
  {
    return sitelauncher.nextUniqueId++;
  },
  
  getIntPref : function(id)
  {
    var val = sitelauncher.prefValues[id];
    if(typeof val === 'number')
    {
      return val;
    }

    return (sitelauncher.prefValues[id] = sitelauncher.prefs.getIntPref(id));
  },
  
  getBoolPref : function(id)
  {
    var val = sitelauncher.prefValues[id];

    if(typeof val === "boolean")
    {
      return val;
    }

    return (sitelauncher.prefValues[id] = sitelauncher.prefs.getBoolPref(id));
  },
  
  getCharPref : function(id)
  {
    var val = sitelauncher.prefValues[id];
    if(typeof val == "string")
    {
      return val;
    }
    else
    {
      return (sitelauncher.prefValues[id] = sitelauncher.prefs.getCharPref(id));
    }
  },
 
  getSiteByKey : function(key, fallback)
  {
    var site = sitelauncherHashTables.get('siteByKey', key);
    
    // fallback to case insensitive if no match found
    if(fallback && site === null)
    {
      site = sitelauncherHashTables.getCaseInsensitive('siteByKey', key);
    }

    return site;
  },
    
  getSiteById : function(id)
  {
    return sitelauncherHashTables.get('siteById', id);
  },
      
  getSiteByUrl : function(url)
  {
    for(var i = 0; i < sitelauncher.sites.length; i++)
    {
      if(sitelauncher.sites[i].url === url)
      {
        return sitelauncher.sites[i];
      }
    }
  },
  
  getSitesByGroup : function(groupId)
  {
    var sites = [];
    for(var i = 0; i < sitelauncher.sites.length; i++)
    {
      if(sitelauncher.sites[i].groupId === groupId)
        sites.push(sitelauncher.sites[i]);
    }

    return sites;
  },
  
  getGroupById : function(id)
  {
    for(var x = 0; x < sitelauncher.groups.length; x++)
    {
      if(sitelauncher.groups[x].id === id)
      {
        return sitelauncher.groups[x];
      }
    }
    
    return false;
  },
  
  getLinkType : function(url)
  {
    if(url.match(/^\s*javascript:/i)) 
    {
      return sitelauncher.linkTypes.javascript;  
    }     
    else if(url.match(/^\s*launchmod:/i))
    {
      return sitelauncher.linkTypes.launchmod;
    }
    else if(url.match(/^\s*search:/i))
    {
      return sitelauncher.linkTypes.search;
    }
    else if(url.split('|').length > 1)
    {
      return sitelauncher.linkTypes.multi;
    }
      
    return sitelauncher.linkTypes.standard; 
  },
  
  init : function()
  {
    if(sitelauncher.prefs === null) 
    {
      sitelauncher.prefs = Components.classes["@mozilla.org/preferences-service;1"].
        getService(Components.interfaces.nsIPrefBranch);
    }
  },
  
  load : function(customFile)
  {   
    sitelauncher.init();
  
    if(!customFile)
    {
      var file = Components.classes["@mozilla.org/file/directory_service;1"].
        getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
      file.append("shortcuts.xml");
      var fis = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);
      try
      {
        fis.init(file, 0x01, 0, 0);
      }
      // shortcuts not found in profile. must be first time install, assign some defaults ...
      catch(e if e.name == "NS_ERROR_FILE_NOT_FOUND")
      {
        // see if we have a default set for user's locale
        try
        {
          var locale = sitelauncher.getCharPref('general.useragent.locale');
          
          file = Components.classes["@mozilla.org/file/directory_service;1"].
                          getService(Components.interfaces.nsIProperties).
                            get("ProfD", Components.interfaces.nsIFile);
                                  
          file.append("extensions");
          file.append("{20291fcc-1471-46c8-8213-5911f5ce6d67}");
          file.append("defaults");   
          file.append("shortcuts-" + locale + ".xml");                                            
  
          //file = Components.classes["@mozilla.org/extensions/manager;1"].
          //  getService(Components.interfaces.nsIExtensionManager).
          //    getInstallLocation('{20291fcc-1471-46c8-8213-5911f5ce6d67}').
          //      getItemFile('{20291fcc-1471-46c8-8213-5911f5ce6d67}', "defaults/shortcuts-" + locale + ".xml");
          fis.init(file, 0x01, 0, 0);
        }
        // fallback to default set
        catch(e if e.name == "NS_ERROR_FILE_NOT_FOUND" || 
                   e.name == "NS_ERROR_FILE_UNRECOGNIZED_PATH")
        {
          file = Components.classes["@mozilla.org/file/directory_service;1"].
                          getService(Components.interfaces.nsIProperties).
                            get("ProfD", Components.interfaces.nsIFile);
                                  
          file.append("extensions");
          file.append("{20291fcc-1471-46c8-8213-5911f5ce6d67}");
          file.append("defaults");   
          file.append("shortcuts.xml");                                            
  
          fis.init(file, 0x01, 0, 0);
        }
      }
    }
    else  // customFile: used for import
    {
      var fis = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);      
      fis.init(customFile, 0x01, 0, 0);
    }
 
    try
    {
      var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"].
                 createInstance(Components.interfaces.nsIConverterInputStream);
      is.init(fis, 'UTF-8', 1024, 0x00000);
      var line = {};
      var string = '';
      while(is.readString(4096, line) != 0)
        string += line.value;
        
      is.close();
      fis.close();   
    }
    catch(e)
    {
      
      if(!customFile) 
      {
        sitelauncher.sites = [];  
        sitelauncher.groups = [];     
      }
      
      is.close();
      fis.close();   
     
      return false;
    }

    var parser = new DOMParser();
    var dom = parser.parseFromString(string, "text/xml");
    
    var shortcuts = dom.getElementsByTagName('shortcut');
    var groups = dom.getElementsByTagName('group');
    
    // only assume problem if custom file, as user might actually have 
    // intentionally deleted all of his/her shortcuts
    if(customFile && !shortcuts.length && !groups.length)
    {
      return false;
    }        

    var container = dom.getElementsByTagName('shortcuts')[0];
    if(container.hasAttribute('version'))
    {
      sitelauncher.shortcutSchemaVersion = parseInt(container.getAttribute('version'), 10);
    }
    else
    {
      sitelauncher.shortcutSchemaVersion = 1;
    }
        
    var requiredIconSize = sitelauncher.requiredIconSize;
    if(requiredIconSize !== -1)
    {
  
      var iconSizeName = ['iconDataSmall', 
                          'iconData',
                          'iconDataLarge'][requiredIconSize];
                          
      var iconDefaultName = ['defaultPageIconSmall',
                             'defaultPageIcon',
                             'defaultPageIconLarge'][requiredIconSize];                        
    }

    sitelauncher.sites = [];
    for(var i = 0; i < shortcuts.length; i++)
    {
      var shortcut = shortcuts[i]; 
      var groupId = sitelauncherUtilsDom.getIntAttr(shortcut, 'groupId', 0); // if no group id default to main (0)
      var url = sitelauncherUtilsDom.getStrAttrTrusted(shortcut, 'url');

      var site = {'id':sitelauncher.getUniqueId(),
                  'key':sitelauncherUtilsDom.getStrAttrTrusted(shortcut, 'key'), 
                  'title':sitelauncherUtilsDom.getStrAttrTrusted(shortcut, 'title'), 
                  'url':url, 
                  'type':sitelauncher.getLinkType(url),
                  'groupId':groupId}; 
      
      if(requiredIconSize !== -1 && sitelauncher.shortcutSchemaVersion > 1)
      {
        // only get icons in required size to conserve memory
        site[iconSizeName] = sitelauncherUtilsDom.getStrAttr(shortcut, iconSizeName, sitelauncher[iconDefaultName]);
      }
      else
      {
        // get all sizes
        site['iconDataSmall'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'iconDataSmall', sitelauncher.defaultPageIconSmall);
        site['iconData'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'iconData', sitelauncher.defaultPageIcon);
        site['iconDataLarge'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'iconDataLarge', sitelauncher.defaultPageIconLarge);
        site['originalIconDataSmall'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'originalIconDataSmall', '');
        site['originalIconData'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'originalIconData', '');
        site['originalIconDataLarge'] = sitelauncherUtilsDom.getStrAttr(shortcut, 'originalIconDataLarge', ''); 
        site['isUsingCustomIcon'] = sitelauncherUtilsDom.getBoolAttr(shortcut, 'isUsingCustomIcon', false);       
      }
      
      sitelauncher.sites.push(site);
    }
    
    var groups = dom.getElementsByTagName('group');
    var groupIds = [];
    sitelauncher.groups = [];
    for(var i = 0; i < groups.length; i++)
    {
      var id = sitelauncherUtilsDom.getIntAttrTrusted(groups[i], 'id');
      var title = sitelauncherUtilsDom.getStrAttrTrusted(groups[i], 'title');
      sitelauncher.groups.push({'id' : id, 'title' : title});
      groupIds.push(id);
    }
    
    // main group (0) must always be present
    if(groupIds.indexOf(0) === -1) 
    {
      sitelauncher.groups.unshift({'id' : 0, 'title' : sitelauncherUtilsMisc.getLocaleString("mainGroupDefaultTitle")});
    }
    
    sitelauncherHashTables.clearAll();
    sitelauncherHashTables.create('siteByKey', sitelauncher.sites, 'key');
    sitelauncherHashTables.create('siteById', sitelauncher.sites, 'id');
    sitelauncherHashTables.create('groupById', sitelauncher.groups, 'id');    
    
    sitelauncherSiteIcons.init();
    
    return true;  
  },

  save : function(e, customFile)
  {
    var doc = document.implementation.createDocument("", "", null);
    
    var sites = sitelauncher.sites;
    var shortcuts = document.createElement('shortcuts');
    shortcuts.setAttribute('version', sitelauncher.shortcutSchemaVersion);
    
    // save shortcuts
    for(var i = 0; i < sites.length; i++)
    {
      var shortcut = document.createElement('shortcut');
      shortcut.setAttribute('key', sites[i].key ? sites[i].key : '');
      shortcut.setAttribute('title', sites[i].title ? sites[i].title : '');
      shortcut.setAttribute('url', sites[i].url ? sites[i].url : '');
      shortcut.setAttribute('groupId', sites[i].groupId ? sites[i].groupId : 0);
      shortcut.setAttribute('iconData', sites[i].iconData ? 
        sites[i].iconData : sitelauncher.defaultPageIcon);
      shortcut.setAttribute('iconDataSmall', sites[i].iconDataSmall ? 
        sites[i].iconDataSmall : sitelauncher.defaultPageIconSmall);
      shortcut.setAttribute('iconDataLarge', sites[i].iconDataLarge ? 
        sites[i].iconDataLarge : sitelauncher.defaultPageIconLarge);       
      shortcut.setAttribute('originalIconData', sites[i].originalIconData  ? 
        sites[i].originalIconData : '');                            
      shortcut.setAttribute('originalIconDataSmall', sites[i].originalIconDataSmall  ? 
        sites[i].originalIconDataSmall : '');   
      shortcut.setAttribute('originalIconDataLarge', sites[i].originalIconDataLarge  ? 
        sites[i].originalIconDataLarge : '');      
      shortcut.setAttribute('isUsingCustomIcon', typeof sites[i].isUsingCustomIcon === "boolean"  ? 
        (sites[i].isUsingCustomIcon ? "true" : "false") : '');         
                
      shortcuts.appendChild(shortcut);
    }

    // save groups
    for(var i = 0; i < sitelauncher.groups.length; i++)
    {
      var group = document.createElement('group');
      group.setAttribute('id', sitelauncher.groups[i].id);
      group.setAttribute('title', sitelauncher.groups[i].title);
      shortcuts.appendChild(group);
    }

    doc.appendChild(shortcuts);
    
    var ser = new XMLSerializer();
    var str = ser.serializeToString(doc, "text/xml");
    
    var id = "{20291fcc-1471-46c8-8213-5911f5ce6d67}";
    var file;
    
    if(!customFile)
    {
      file = Components.classes["@mozilla.org/file/directory_service;1"].
               getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
      file.append("shortcuts.xml");
    }
    else
    {
      file = customFile;
    }    

    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].
      createInstance(Components.interfaces.nsIFileOutputStream);
    fos.init(file, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
    
    var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
               createInstance(Components.interfaces.nsIConverterOutputStream);
    os.init(fos, "UTF-8", 0, 0x0000);
    os.writeString("<?xml version='1.0' encoding='UTF-8'?>" + str);
    os.close();

    fos.close();

    // let launcher.js instances know we've made some changes
    if(!customFile)
    {    
      sitelauncher.prefs.setIntPref('extensions.sitelauncher.modified', new Date().getTime());
    }
  },
  
  exportFile : function(file)
  {
    try
    {
      sitelauncher.save(null, file);
    }
    catch(e)
    {
    }
  },

  setRequiredIconSize : function(size)
  {
    sitelauncher.requiredIconSize = size;
  },

  updateSiteItemIcon : function(site)
  {
    // use icon data if we already have it from another shortcut
    for(var i = 0; i < sitelauncher.sites.length; i++)
    {
      if(sitelauncher.sites[i].url === site.url && 
         !sitelauncherSiteIcons.isSiteIconBlank(sitelauncher.sites[i]))
      {
        site.iconData = sitelauncher.sites[i].iconData;
        site.iconDataSmall = sitelauncher.sites[i].iconDataSmall;
        site.iconDataLarge = sitelauncher.sites[i].iconDataLarge;        
        return;
      }
    }

    sitelauncherSiteIcons.insertSiteIcons(site);
  },

  add : function(key, title, url)
  {
    if(url === '' || url === null || key.length > 1)
    {
      return false;
    }
    
    var site = sitelauncher.getSiteByKey(key);
    
    // don't allow adding site under existing key
    if(site)
    {
      return false;
    }

    site = {id:sitelauncher.getUniqueId(),
            key:key, 
            title:title, 
            url:url, 
            type:sitelauncher.getLinkType(url),
            groupId:0,
            iconData:sitelauncher.defaultPageIcon,
            iconDataSmall:sitelauncher.defaultPageIconSmall,
            iconDataLarge:sitelauncher.defaultPageIconLarge}

    sitelauncher.sites.push(site);
      
    if(site.key !== '')
    {
      sitelauncherHashTables.insert('siteByKey', site.key, site);
    }
    sitelauncherHashTables.insert('siteById', site.id, site);

    sitelauncher.updateSiteItemIcon(site);

    return site;
  },

  revise : function(id, key, title, url)
  {
    var site = sitelauncher.getSiteById(id);
    
    if(!site)
    {
      return false;
    }
    
    var type = sitelauncher.getLinkType(url);      
    var needsIconUpdated = false;
    
    // get new icon if url changed or current is missing
    if(site.url.replace(/^\s+|\s+$/g, '') != url.replace(/^\s+|\s+$/g, '') || 
       sitelauncherSiteIcons.isSiteIconBlank(site))
    {
      needsIconUpdated = true;
      site.iconData = sitelauncher.defaultPageIcon;
      site.iconDataSmall = sitelauncher.defaultPageIconSmall;
      site.iconDataLarge = sitelauncher.defaultPageIconLarge;  
      site.originalIconData = "";
    }
      
    if(key != site.key)
    {
      sitelauncherHashTables.remove('siteByKey', site.key);
        
      site.key = key;
        
      if(site.key !== '')
      {
        sitelauncherHashTables.insert('siteByKey', site.key, site);
      } 
    }
      
    site.title = title;
    site.url = url;      
    site.type = type;
    
    if(needsIconUpdated)
    {
      sitelauncher.updateSiteItemIcon(site);
    }
    
    return site;    
  },

  removeById : function(id)
  {
    var site = sitelauncher.getSiteById(id);
    var index = sitelauncher.getIndexForSite(site);
    
    sitelauncher.sites.splice(index, 1); 
    sitelauncherHashTables.remove('siteByKey', site.key);
    sitelauncherHashTables.remove('siteById', site.id);
  },
      
  sort : function(index)
  {
    switch(index)
    {
      // no sort
      case 0:
        return;
      break;
      // sort by key
      case 1:
        sitelauncher.sites.sort(function(a,b){
        
        if(a.key === b.key)
        {
          return 0;
        }
        
        // order upper/lower case
        if(a.key.toLowerCase() === b.key)
        {
          return 1;
        } 
        
        var x = a.key.toLowerCase();
        var y = b.key.toLowerCase();

        if(x === '' && y !== '')
        {
          return 1;
        }
        else if(x !== '' && y === '')
        {
          return -1;
        }
            
        return ((x < y) ? -1 : ((x > y) ? 1 : 0)) });
      break;
      // sort by title
      case 2:
        sitelauncher.sites.sort(function(a,b){var x = a.title.toLowerCase(); var y = b.title.toLowerCase();
        return ((x < y) ? -1 : ((x > y) ? 1 : 0)) });
      break;
    }
  },

  changeGroupById : function(id, groupId)
  {
    var site = sitelauncher.getSiteById(parseInt(id, 10));
    
    if(site)
    {
      site.groupId = parseInt(groupId, 10);
    }
  },  
  
  addGroup : function(name)
  {
    var most = 0;
    for(var i = 0; i < sitelauncher.groups.length; i++)
    {
      if(sitelauncher.groups[i].id > most)
        most = sitelauncher.groups[i].id;
    }
    most++; // next unique id
    
    if(name == "")
    {
      name = 'Untitled';
    }
    
    sitelauncher.groups.push({'title' : name, 'id' : most});
    
    return most;
  },
  
  deleteGroup : function(id)
  {
    for(var i = 0; i < sitelauncher.groups.length; i++)
    {
      if(sitelauncher.groups[i].id === id)
      {
        sitelauncher.groups.splice(i, 1);
        break;
      }
    }

    sitelauncher.deleteGroupContents(id);
  },
  
  deleteGroupContents : function(id)
  {
    var sites = sitelauncher.getSitesByGroup(id);
    for(var x = 0; x < sites.length; x++)
    {
      sitelauncher.removeById(sites[x].id);
    }
  },
  
  editGroup : function(id, title)
  {
    for(var i = 0; i < sitelauncher.groups.length; i++)
    {
      if(sitelauncher.groups[i].id === id)
      {
        sitelauncher.groups[i].title = title;
        break;
      }
    }
  },

  moveToAfterById : function(moveId, afterId)
  {
    if(moveId === afterId)
    {
      return false;
    }
    
    var site = sitelauncher.getSiteById(moveId);
    var index = sitelauncher.getIndexForSite(site);
    
    sitelauncher.sites.splice(index, 1); 
    
    // move item to top of its group
    if(afterId === -1)
    {
      for(var x = 0; x < sitelauncher.sites.length; x++)
      {
        if(sitelauncher.sites[x].groupId === site.groupId)
        {
          sitelauncher.sites.splice(x, 0, site);
          return true;
        }
      }
    }
    // move item after other item
    else
    {
      for(var i = 0; i < sitelauncher.sites.length; i++)
      {
        if(sitelauncher.sites[i].id === afterId)
        {
          sitelauncher.sites.splice(i + 1, 0, site);
          return true;   
        }
      }    
    }
    
    // if we reached this point, item wasn't
    // moved to new position; therefore revert 
    // to old position
    sitelauncher.sites.splice(index, 0, site);
    
    return false;
  },
  
  getIndexForSite : function(site)
  {
    for(var i = 0; i < sitelauncher.sites.length; i++)
    {
      if(sitelauncher.sites[i].id === site.id)
      {
        return i;    
      }
    }  
  },  

  moveGroupToAfter : function(moveId, afterId)
  {
    var group = sitelauncher.getGroupById(moveId);
    
    if(!group)
    {
      return false;
    }
    
    // rm old position
    for(var i = 0; i < sitelauncher.groups.length; i++)
    {
      if(sitelauncher.groups[i].id === moveId)
      {
        sitelauncher.groups.splice(i, 1);
      }
    }    
    
    // move to top
    if(afterId === -1)
    {
      sitelauncher.groups.splice(0, 0, group);
      
      return true;
    }
    else
    {
      for(var i = 0; i < sitelauncher.groups.length; i++)
      {
        if(sitelauncher.groups[i].id === afterId)
        {
          sitelauncher.groups.splice(i + 1, 0, group);   
            
          return true;   
        }
      }         
    }
    
    return false;
  },
};
